@@ -374,7 +374,7 @@ objClass(SkeletonObjectType const objType) {
 
 #if HAVE_PARSE_PRINTF_FORMAT
 static void
-validateParsePrintfFLag(int                const printfConversion,
+validateParsePrintfFlag(int                const printfConversion,
                         SkeletonObjectType const ctyp,
                         const char **      const errorP) {
 /*----------------------------------------------------------------------------
@@ -436,6 +436,7 @@ validateParsePrintfFLag(int                const printfConversion,
 
 
 
+#if HAVE_PARSE_PRINTF_FORMAT
 static void
 validateFormatWithPpf(const char *       const format,
                       SkeletonObjectType const ctyp,
@@ -466,7 +467,7 @@ validateFormatWithPpf(const char *       const format,
         break;
 
     case 1:
-        validateParsePrintfFLag(printfConversion[0], ctyp, errorP);
+        validateParsePrintfFlag(printfConversion[0], ctyp, errorP);
         break;
 
     default:
@@ -475,6 +476,7 @@ validateFormatWithPpf(const char *       const format,
         break;
     }
 }
+#endif
 
 
 
